package jp.co.kayo.android.localplayer.ds.boxnet;

import java.io.IOException;
import java.util.Iterator;

import com.box.androidlib.Box;
import com.box.androidlib.DAO.BoxFile;
import com.box.androidlib.DAO.BoxFolder;
import com.box.androidlib.DAO.User;
import com.box.androidlib.ResponseListeners.GetAccountInfoListener;
import com.box.androidlib.ResponseListeners.GetAccountTreeListener;

import android.app.IntentService;
import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.Build;
import android.preference.PreferenceManager;
import android.widget.Toast;

public class MediaScanner extends IntentService {
    boolean isProc = false;
    boolean isStop = false;
    SharedPreferences pref;
    BoxnetHelper boxnetHelper;
    private String[] mediaextensions = null;
    private String[] videoextensions = null;
    FileNameRetriever retriver = new FileNameRetriever();
    String editFolderPattern = null;
    String editFilePattern = null;

    public MediaScanner() {
        super("MediaScanner");
    }

    public MediaScanner(String name) {
        super(name);
    }

    @Override
    protected void onHandleIntent(Intent intent) {
        pref = PreferenceManager.getDefaultSharedPreferences(this);
        if (Build.VERSION.SDK_INT > 10) {
            StrictHelper.registStrictMode();
        }

        try {
            if (!isProc) {
                if (intent != null && intent.getAction() != null) {
                    isProc = true;
                    if (intent.getAction().equals("fullsync")) {
                        startScan(true);
                    } else if (intent.getAction().equals("sync")) {
                        startScan(false);
                    }
                }
            } else if (intent != null && intent.getAction() != null
                    && intent.getAction().equals("stop")) {
                isStop = true;
            }
        } finally {
            isProc = false;
            stopSelf();
        }
    }

    private String[] getMediaExtensionList() {
        if (mediaextensions == null) {
            String editMediaExt = pref.getString("editMediaExt",
                    SystemConsts.MEDIA_EXT);
            if (editMediaExt.trim().length() > 0) {
                String[] ext = editMediaExt.split(",");
                if (ext != null && ext.length > 0) {
                    String[] ret = new String[ext.length];
                    for (int i = 0; i < ext.length; i++) {
                        ret[i] = ext[i].trim().toLowerCase();
                    }
                    mediaextensions = ret;
                    return mediaextensions;
                }
            }
            mediaextensions = SystemConsts.MEDIA_EXT.split(",");
        }
        return mediaextensions;
    }

    private String[] getVideoExtensionList() {
        if (videoextensions == null) {
            String editVideoExt = pref.getString("editVideoExt",
                    SystemConsts.VIDEO_EXT);
            if (editVideoExt.trim().length() > 0) {
                String[] ext = editVideoExt.split(",");
                if (ext != null && ext.length > 0) {
                    String[] ret = new String[ext.length];
                    for (int i = 0; i < ext.length; i++) {
                        ret[i] = ext[i].trim().toLowerCase();
                    }
                    videoextensions = ret;
                    return videoextensions;
                }
            }
            videoextensions = SystemConsts.VIDEO_EXT.split(",");
        }
        return videoextensions;
    }

    private String getFolderPattern() {
        if (editFolderPattern == null) {
            editFolderPattern = pref.getString("editFolderPattern", "%a/%A")
                    .trim();
            if (editFolderPattern.length() == 0) {
                editFolderPattern = "%a - %T - %t";
            }
        }
        return editFolderPattern;
    }

    private String getFilePattern() {
        if (editFilePattern == null) {
            editFilePattern = pref.getString("editFilePattern", "%T - %a - %t")
                    .trim();
            if (editFilePattern.length() == 0) {
                editFilePattern = "%a/%A";
            }
        }
        return editFilePattern;
    }

    public static boolean isMedia(String fname, String[] exts) {
        int s1 = fname.lastIndexOf('.');
        if (s1 > 0) {
            String ext = fname.substring(s1);
            for (int i = 0; i < exts.length; i++) {
                if (ext.toLowerCase().equals(exts[i])) {
                    return true;
                }
            }
        }
        return false;
    }

    public static boolean isJPG(String fname) {
        int s1 = fname.lastIndexOf('.');
        if (s1 > 0) {
            String ext = fname.substring(s1);
            if (ext.toLowerCase().equals(".jpg")) {
                return true;
            }
        }
        return false;
    }

    public static long scanAlbumArt(Iterator<? extends BoxFile> filesIterator) {
        long albumArt = -1;
        while (filesIterator.hasNext()) {
            BoxFile boxFile = filesIterator.next();
            String name = boxFile.getFileName();
            if (isJPG(name)) {
                albumArt = boxFile.getId();
                if (name.toLowerCase().indexOf("cover") != -1) {
                    break;
                }
            }
        }
        return albumArt;
    }

    private void notifyOn() {
        NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        Notification notification;
        notification = new Notification(R.drawable.ic_stat_name,
                getString(R.string.prg_name), System.currentTimeMillis());
        notification.flags = Notification.FLAG_NO_CLEAR
                | Notification.FLAG_ONGOING_EVENT;

        Intent intent = new Intent(this, StopActivity.class);

        PendingIntent contentIntent = PendingIntent.getActivity(this, 0,
                intent, Intent.FLAG_ACTIVITY_NEW_TASK);
        notification.setLatestEventInfo(this, getString(R.string.prg_name),
                getString(R.string.msg_start), contentIntent);

        nm.notify(R.string.app_name, notification);
    }

    private void notifyOff() {
        NotificationManager nm = (NotificationManager) getSystemService(NOTIFICATION_SERVICE);
        nm.cancel(R.string.app_name);
    }

    private void startScan(boolean fullsync) {
        try {
            isStop = false;
            notifyOn();

            // 初期化
            if (fullsync) {
                getContentResolver().query(MediaConsts.RESET_CONTENT_URI, null,
                        null, null, null);
            }

            boxnetHelper = new BoxnetHelper(this);
            boolean isLoggedIn = boxnetHelper.isLoggdIn();
            if (isLoggedIn) {

                final SharedPreferences pref = getSharedPreferences(
                        SystemConsts.PREFS_FILE_NAME, 0);
                final long space_used = pref.getLong(SystemConsts.INF_SPACE_USED,
                        0);
                final String authToken = boxnetHelper.getAuthToken();

                final Box boxServiceHandler = Box
                        .getInstance(SystemConsts.API_KEY);
                boxServiceHandler.getAccountInfo(authToken,
                        new GetAccountInfoListener() {

                            @Override
                            public void onComplete(final User boxUser,
                                    String status) {
                                if (status
                                        .equals(GetAccountInfoListener.STATUS_GET_ACCOUNT_INFO_OK)
                                        && boxUser != null) {
                                    long current = boxUser.getSpaceUsed();
                                    if (current != space_used) {
                                        startScan(0l, 0);
                                        Editor editor = pref.edit();
                                        editor.putLong(SystemConsts.INF_SPACE_USED, current);
                                        editor.commit();
                                    }
                                    else{
                                        Toast.makeText(boxnetHelper.getContext(), getString(R.string.msg_nothing_update), Toast.LENGTH_LONG).show();
                                    }
                                }
                            }

                            @Override
                            public void onIOException(IOException e) {
                            }
                        });
            }
        } finally {
            notifyOff();
        }
    }

    private void startScan(final long folderId, final int curLevel) {
        final Box box = Box.getInstance(SystemConsts.API_KEY);
        box.getAccountTree(boxnetHelper.getAuthToken(), folderId,
                new String[] { Box.PARAM_ONELEVEL },
                new GetAccountTreeListener() {

                    @Override
                    public void onComplete(BoxFolder boxFolder, String status) {
                        if (!status
                                .equals(GetAccountTreeListener.STATUS_LISTING_OK)) {
                            Toast.makeText(getApplicationContext(),
                                    "There was an error.", Toast.LENGTH_SHORT)
                                    .show();
                            return;
                        }

                        // Folder
                        Iterator<? extends BoxFolder> foldersIterator = boxFolder
                                .getFoldersInFolder().iterator();
                        while (foldersIterator.hasNext()) {
                            BoxFolder subfolder = foldersIterator.next();
                            startScan(subfolder.getId(), curLevel + 1);
                        }

                        // File
                        int track = 0;
                        long albumArt = -1;
                        Iterator<? extends BoxFile> filesIterator = boxFolder
                                .getFilesInFolder().iterator();
                        while (filesIterator.hasNext()) {
                            BoxFile boxFile = filesIterator.next();
                            if (isMedia(boxFile.getFileName(),
                                    getMediaExtensionList())) {
                                track++;
                                if (albumArt == -1) {
                                    albumArt = scanAlbumArt(boxFolder
                                            .getFilesInFolder().iterator());
                                }
                                FileMetaInfo ret = retriver.get(boxnetHelper,
                                        boxFile, getFolderPattern(),
                                        getFilePattern());
                                if (ret != null) {
                                    ret.albumArt = albumArt;
                                    ret.track = Integer.toString(track);
                                    boxnetHelper.registMetaInfo(ret);
                                }
                            } else {

                            }
                        }
                    }

                    @Override
                    public void onIOException(final IOException e) {
                        Toast.makeText(getApplicationContext(),
                                "Failed to get tree - " + e.getMessage(),
                                Toast.LENGTH_LONG).show();
                    }
                });

    }
}
